package main;

import java.util.ArrayList;

import kmeans.KMmain;

public class Cluster {
	
	protected ArrayList<Integer> punkte;
	protected double[] center;
	protected double mean;
	
	
	public Cluster(){
		punkte = new ArrayList<Integer>();
	}
	
	public Cluster(double[] cen){
		punkte = new ArrayList<Integer>();
		center = cen;
	}
	
	public Cluster(ArrayList<Integer> clu) {
		this.setPunkte(clu);
	}

	
	// berechnet die Wahrscheinlichkeit eines punktes unter normalverteilung, bei bekannter distanz und standardabweichung.
	/*public double pdf(double dist,double dev){
		double reto=0;
		if(dev==0){
			return 1;
		}
		//1/((2 pi)^(1/2)) ca 0.398942
		reto=0.398942*(1/dev)*Math.exp(-(dist*dist)/(2*dev*dev));
		return reto;
	}*/
	
	public double[] newCentre(DipKM boo) {
		int dim = this.get(boo, 0).length;
		double[] center = new double[dim];
		
		for(int i=0;i<this.size();i++){
			double[] now = this.get(boo, i);
			for(int j=0;j<dim;j++){
				center[j]=center[j]+now[j];
			}
		}
		for(int j=0;j<dim;j++){
			center[j]=center[j]/this.size();
		}
		return center;
	}
	
	public void calculateCenterAuto(DipKM boo){
		this.setCenter(this.newCentre(boo));
	}
	
	public void turn(double alpha, DipKM boo){
		alpha=0.01745*alpha;
		ArrayList<double[]> data = this.getClusterData(boo);
		double sinalpha = Math.sin(alpha);
		double cosalpha = Math.cos(alpha);
		if(data.get(0).length!=2){
			System.out.println("Mgl. Problem: Falsche Dim!");
		}
		for(int i=0;i<data.size();i++){
			double[] old = data.get(i);
			double[] ado = new double[old.length];
			ado[0]=old[0]*cosalpha-old[1]*sinalpha;
			ado[1]=old[0]*sinalpha+old[1]*cosalpha;
			old[0]=ado[0];
			old[1]=ado[1];
		}
	}
	
	public ArrayList<Integer> getPunkte() {
		return punkte;
	}

	private void setPunkte(ArrayList<Integer> neo){
		punkte = neo;
	}
	
	public void clear(){
		this.punkte.clear();
	}
	
	public int size(){
		return this.getPunkte().size();
	}
	
	public void add(int a){
		this.punkte.add(a);
	}

	public void add(ArrayList<Integer> ado) {
		for(int i=0;i<ado.size();i++){
			this.add(ado.get(i));
		}
	}
	
	public void setCenter(double[] cen) {
		this.center=cen;
	}
	
	public double[] getCenter() {
		return center;
	}
	
	public void setMean(double mean){
		this.mean=mean;
	}
	
	public double getMean(){
		return mean;
	}
	
	public void combine(Cluster ada){
		for(int i=0;i<ada.size();i++){
			this.add(ada.getPunkte());
		}
		ada.clear();
	}
	
	public void remove(Integer a){
		this.punkte.remove(a);
	}
	
	public double[] get(DipKM boo, int index){
		return boo.getDatapoints().get(this.getPunkte().get(index));
	}
	
	public void purePrint(){
		for(int i=0;i<this.size();i++){
			System.out.println(this.getPunkte().get(i));
		}
	}

	public void print(DipKM boo) {
		for(int i=0;i<this.size();i++){
			BenUtil.print(this.get(boo, i));
		}
	}
	
	public ArrayList<double[]> getClusterData(DipKM boo){
		ArrayList<double[]> data = new ArrayList<double[]>();
		for(int i=0;i<this.size();i++){
			data.add(this.get(boo, i));
		}
		return data;
	}
	
	public ArrayList<double[]> copyClusterData(DipKM boo){
		ArrayList<double[]> data = new ArrayList<double[]>();
		for(int i=0;i<this.size();i++){
			data.add(this.get(boo, i).clone());
		}
		return data;
	}
	
	public static double[][] kMeansResultCenter(ArrayList<double[]> datacopy, int k) {
		KMmain te = new KMmain(datacopy);
		te.clustering(k, 100, null);
		double[][] lab = te.getCentroids();
		return lab;
	}
	
	public static double[] getDirection(ArrayList<double[]> datacopy){ 
		double[][] in = Cluster.kMeansResultCenter(datacopy, 2);		
		double[] vect = new double[datacopy.get(0).length];
		for(int i=0;i<vect.length;i++){
			vect[i]=in[0][i]-in[1][i];
		}
		BenUtil.normalize(vect);
		return vect;
	}
	
	public double[] getProjection(ArrayList<double[]> datacopy2, double[] direction) {
		double[] proj = new double[datacopy2.size()];
		for(int i=0;i<proj.length;i++){
			proj[i] = BenUtil.dot(direction, datacopy2.get(i));
		}
		return proj;
	}
	
	private void stretchInDirectionBy(ArrayList<double[]> datacopy, double[] direction, double str, double[] proj) {
		int maxI = BenUtil.IndexMax(proj);
		int minI = BenUtil.IndexMin(proj);
		double min = proj[minI];
		double max = proj[maxI];
		
		for(int i=0;i<datacopy.size();i++){
			double[] now = datacopy.get(i);
			double nol = (proj[i]-min)/(max-min)*str-proj[i];
			double[] offset = BenUtil.mult(direction,nol);
			now = BenUtil.add(now,offset);
		}
	}
	

	
	public ArrayList<double[]> simpleKMDipTrans(DipKM thea) {
		ArrayList<double[]> datacopy = this.copyClusterData(thea);
		ArrayList<double[]> datacopy2 = this.copyClusterData(thea);
		this.calculateCenterAuto(thea);
		double[] center = this.getCenter();
		
		for(int i=0;i<thea.getDimension();i++){
			double[] direction = Cluster.getDirection(datacopy);
								
			double[] proj = this.getProjection(datacopy2,direction);
			double[] dip = DipKM.DipTest(proj);
			
			double str = (dip[0]);
			this.stretchInDirectionBy(datacopy,direction,str,proj);
			this.stretchInDirectionBy(datacopy2,direction, 0, proj);
		}
		BenUtil.shiftBy(datacopy, center);
		return datacopy;
	}





}
